;*************************************************************** 
;* 
;* Kls interrupt
;* 
;* A program az EXTINT0 (PD0 pin) interruptra megvltoztatja a PC0 LED llapott.
;* A fprogram egy msik LED-et (PC2) villogtat 
;* 
;*************************************************************** 
;* 
;*  Kszitette:
;*  Csoport:
;*  Dtum:
;* 
;*************************************************************** 
 
.include "m128def.inc"	; ATmega128 defincis fjl
 
;*************************************************************** 
;* Konstansok 
;***************************************************************  
 .equ	LED1nr	= 2
 .equ	LED1mask = 1<<LED1nr

 .equ	LED2nr	= 0
 .equ	LED2mask = 1<<LED2nr

 .equ	BUTnr	= 0

;*************************************************************** 
;* Vltozk
;*************************************************************** 
.def	mpr1 = R16		; ltalnos felhasznls regiszterek
.def	mpr2 = R17
.def	cnt1 = R18		; Szmllk a ksleltetshez
.def	cnt2 = R19


 
;*************************************************************** 
;* Megszakts vektorok
;*************************************************************** 
.cseg 
.org 0x0000      ; Kdszegmens kezdcme

		jmp		init		; Reset vektor 
		jmp		ext0_it		; EXTINT0 Handler
		jmp		dummy		; EXTINT1 Handler
		jmp		dummy		; EXTINT2 Handler
		jmp		dummy		; EXTINT3 Handler
		jmp		dummy		; EXTINT4 Handler (INT gomb)
		jmp		dummy		; EXTINT5 Handler
		jmp		dummy		; EXTINT6 Handler
		jmp		dummy		; EXTINT7 Handler
		jmp		dummy		; Timer2 Compare Match Handler 
		jmp		dummy		; Timer2 Overflow Handler 
		jmp		dummy		; Timer1 Capture Event Handler 
		jmp		dummy		; Timer1 Compare Match A Handler 
		jmp		dummy		; Timer1 Compare Match B Handler 
		jmp		dummy		; Timer1 Overflow Handler 
		jmp		dummy		; Timer0 Compare Match Handler 
		jmp		dummy		; Timer0 Overflow Handler 
		jmp		dummy		; SPI Transfer Complete Handler 
		jmp		dummy		; USART0 RX Complete Handler 
		jmp		dummy		; USART0 Data Register Empty Hanlder 
		jmp		dummy		; USART0 TX Complete Handler 
		jmp		dummy		; ADC Conversion Complete Handler 
		jmp		dummy		; EEPROM Ready Hanlder 
		jmp		dummy		; Analog Comparator Handler 
		jmp		dummy		; Timer1 Compare Match C Handler 
		jmp		dummy		; Timer3 Capture Event Handler 
		jmp		dummy		; Timer3 Compare Match A Handler 
		jmp		dummy		; Timer3 Compare Match B Handler 
		jmp		dummy		; Timer3 Compare Match C Handler 
		jmp		dummy		; Timer3 Overflow Handler 
		jmp		dummy		; USART1 RX Complete Handler 
		jmp		dummy		; USART1 Data Register Empty Hanlder 
		jmp		dummy		; USART1 TX Complete Handler 
		jmp		dummy		; Two-wire Serial Interface Handler 
		jmp		dummy		; Store Program Memory Ready Handler 
	
;*****************************************************************************
;* Inicializls
;*****************************************************************************

;***** Stack inicializlsa *****

init:	ldi		mpr1,LOW(RAMEND)	; RAMEND = RAM vgcme
		out		SPL,mpr1			; (ld."m128def.inc") 
		ldi		mpr1,HIGH(RAMEND)
		out		SPH,mpr1

;*****************************************************************************
;* Fprogram
;*****************************************************************************		

main:
		sbi		DDRC,LED1mask	; C port 2-es lbt kimenetre lltjuk
		sbi		DDRC,LED2mask	; C port 0-s lbt kimenetre lltjuk

		ldi		mpr2,LED1mask	; Set mpr2 for LED1

		ldi		cnt1,0xFF		; Betjuk az idztseket
		ldi		cnt2,0x10


		ldi		mpr1,0x00
		out		DDRD,mpr1		; D port bemenett lltsa

		sbi		PORTD,BUTnr		; Pull up Port D 0 pin (EXTINT0)

		ldi		mpr1,0x02
		sts		EICRA, mpr1		; belltjuk EXTINT0-t leszll gra

		ldi		mpr1,0x01
		out		EIMSK,mpr1		; Engedlyezzk az EXTINT0 megszaktst
		sei						; Megszaktsok engedlyezse


loop:
		in		mpr1,PORTC		; Port beolvassa
		eor		mpr1,mpr2		; Adott portlb invertlsa
		out		PORTC,mpr1		; Invertlt rtk portra kldse
		call	delay			; Ksleltets
		jmp		loop			; Vgtelen hurok

		
;*****************************************************************************
;* Szubrutinok
;*****************************************************************************
; Ksleltets (bemenetek: cnt1 s cnt2)

delay:
		push	mpr1
		push	cnt1
		push	cnt2
del1:
		ldi		mpr1,0xff
del2:
		dec		mpr1
		brne	del2
		dec		cnt1
	 	brne	del1
		dec		cnt2
		brne	del1
		pop		cnt2
		pop		cnt1
		pop		mpr1
		ret	

;*****************************************************************************
;* Megszakts kezel rutinok
;*****************************************************************************

ext0_it:
		push	mpr1			; mpr1 s mpr2 elmentse a veremtrba
		push	mpr2
		in		mpr1,PORTC		; Port C beolvassa
		ldi		mpr2,LED2mask	; mpr2-be LED2mask
		eor		mpr1,mpr2		; Kizr vagy (invertljuk a portlb rtkt)
		out		PORTC,mpr1		; Kirakjukt a portra
		pop		mpr2			; mpr1 s mpr2 visszalltsa a veremtrbl
		pop		mpr1
		
		reti;

;*****************************************************************************
;* Adatok
;*****************************************************************************
